@@ -100,7 +100,7 @@ class Downloader:
         self.dl_retry_delay = dl_retry_delay
         self.lta_retry_delay = lta_retry_delay
         self.lta_timeout = lta_timeout
-        self.chunk_size = 2 ** 20  # download in 1 MB chunks by default
+        self.chunk_size = 2**20  # download in 1 MB chunks by default
 
     def download(self, id, directory=".", *, stop_event=None):
         """Download a product.
@@ -152,9 +152,9 @@ class Downloader:
 
     def _download_with_node_filter(self, id, directory, stop_event):
         product_info = self.api.get_product_odata(id)
-        product_path = Path(directory) / (product_info["title"] + ".SAFE")
-        product_info["node_path"] = "./" + product_info["title"] + ".SAFE"
-        manifest_path = product_path / "manifest.safe"
+        product_path = Path(directory) / (product_info["product_root_dir"])
+        product_info["node_path"] = "./" + product_info["product_root_dir"]
+        manifest_path = product_path / product_info["manifest_name"]
         if not manifest_path.exists() and self.trigger_offline_retrieval(id):
             raise LTATriggered(id)
         manifest_info, _ = self.api._get_manifest(product_info, manifest_path)
@@ -769,7 +769,7 @@ class Downloader:
             node_info["md5"] = dataobj_info["md5"]
         if "sha3-256" in dataobj_info:
             node_info["sha3-256"] = dataobj_info["sha3-256"]
-        node_info["node_path"] = dataobj_info["href"]
+        node_info["node_path"] = path
         # node_info["parent"] = product_info
 
         return node_info
