@@ -510,6 +510,12 @@ class SentinelAPI:
             response = self.session.get(url)
         self._check_scihub_response(response)
         values = _parse_odata_response(response.json()["d"])
+        if values["title"].startswith("S3"):
+            values["manifest_name"] = "xfdumanifest.xml"
+            values["product_root_dir"] = values["title"] + ".SEN3"
+        else:
+            values["manifest_name"] = "manifest.safe"
+            values["product_root_dir"] = values["title"] + ".SAFE"
         values["quicklook_url"] = self._get_odata_url(id, "/Products('Quicklook')/$value")
         return values
 
@@ -944,7 +950,7 @@ class SentinelAPI:
 
         return corrupt
 
-    def _checksum_compare(self, file_path, product_info, block_size=2 ** 13):
+    def _checksum_compare(self, file_path, product_info, block_size=2**13):
         """Compare a given MD5 checksum with one calculated from a file."""
         if "sha3-256" in product_info:
             checksum = product_info["sha3-256"]
@@ -1066,7 +1072,7 @@ class SentinelAPI:
 
     def _path_to_url(self, product_info, path, urltype=None):
         id = product_info["id"]
-        title = product_info["title"]
+        root_dir = product_info["product_root_dir"]
         path = "/".join(["Nodes('{}')".format(item) for item in path.split("/")])
         if urltype == "value":
             urltype = "/$value"
@@ -1077,14 +1083,15 @@ class SentinelAPI:
         elif urltype is None:
             urltype = ""
         # else: pass urltype as is
-        return self._get_odata_url(id, f"/Nodes('{title}.SAFE')/{path}{urltype}")
+        return self._get_odata_url(id, f"/Nodes('{root_dir}')/{path}{urltype}")
 
     def _get_manifest(self, product_info, path=None):
         path = Path(path) if path else None
-        url = self._path_to_url(product_info, "manifest.safe", "value")
+        manifest_name = product_info["manifest_name"]
+        url = self._path_to_url(product_info, manifest_name, "value")
         node_info = product_info.copy()
         node_info["url"] = url
-        node_info["node_path"] = "./manifest.safe"
+        node_info["node_path"] = f"./{manifest_name}"
         del node_info["md5"]
 
         if path and path.exists():
@@ -1093,7 +1100,7 @@ class SentinelAPI:
             node_info["size"] = len(data)
             return node_info, data
 
-        url = self._path_to_url(product_info, "manifest.safe", "json")
+        url = self._path_to_url(product_info, manifest_name, "json")
         with self.dl_limit_semaphore:
             response = self.session.get(url)
         self._check_scihub_response(response)
